from abc import ABC, abstractmethod
from typing import Optional

from starlette.middleware import Middleware


class MiddlewareFactoryABC(ABC):
    priority: int = 100

    @abstractmethod
    def create(self) -> Optional[Middleware]:
        """Optionally create middleware for the application - used for things like cors"""



import os
from typing import List

CORS_ALLOWED_ORIGINS = "CORS_ALLOWED_ORIGINS"


def get_allowed_origins() -> List[str]:
    allowed_origins = os.environ.get(CORS_ALLOWED_ORIGINS) or ""
    result = (a.strip() for a in allowed_origins.split(","))
    result = [a for a in result if a]
    return result



from dataclasses import dataclass, field
from typing import Optional, List

from starlette.middleware import Middleware
from starlette.middleware.cors import CORSMiddleware

from servey.cors import get_allowed_origins
from servey.servey_starlette.middleware.middleware_factory_abc import (
    MiddlewareFactoryABC,
)


@dataclass
class CORSMiddlewareFactory(MiddlewareFactoryABC):
    allow_origins: List[str] = field(default_factory=get_allowed_origins)

    def create(self) -> Optional[Middleware]:
        if self.allow_origins:
            return Middleware(
                CORSMiddleware,
                allow_origins=self.allow_origins,
                allow_methods="*",
                allow_headers="*",
                allow_credentials=True,
            )
